@@ -89,14 +89,14 @@ soup_request_data_send (SoupRequest   *request,
 	comma = strchr (start, ',');
 	if (comma && comma != start) {
 		/* Deal with MIME type / params */
-		if (comma > start + BASE64_INDICATOR_LEN && !g_ascii_strncasecmp (comma - BASE64_INDICATOR_LEN, BASE64_INDICATOR, BASE64_INDICATOR_LEN)) {
+		if (comma >= start + BASE64_INDICATOR_LEN && !g_ascii_strncasecmp (comma - BASE64_INDICATOR_LEN, BASE64_INDICATOR, BASE64_INDICATOR_LEN)) {
 			end = comma - BASE64_INDICATOR_LEN;
 			base64 = TRUE;
 		} else
 			end = comma;
 
 		if (end != start)
-			data->priv->content_type = uri_decoded_copy (start, end - start, NULL);
+			data->priv->content_type = soup_uri_decoded_copy (start, end - start, NULL);
 	}
 
 	memstream = g_memory_input_stream_new ();
@@ -106,7 +106,8 @@ soup_request_data_send (SoupRequest   *request,
 
 	if (*start) {
 		int decoded_length = 0;
-		guchar *buf = (guchar *) uri_decoded_copy (start, strlen (start), &decoded_length);
+		guchar *buf = (guchar *) soup_uri_decoded_copy (start, strlen (start),
+								&decoded_length);
 
 		if (base64)
 			buf = g_base64_decode_inplace ((gchar*) buf, &data->priv->content_length);
