@@ -19,7 +19,7 @@ typedef struct {
 typedef struct {
 	TestRequest requests[3];
 	guint final_status;
-	guint request_api_final_status;
+	const char *bugref;
 } TestCase;
 
 static TestCase tests[] = {
@@ -27,95 +27,95 @@ static TestCase tests[] = {
 
 	{ { { "GET", "/301", 301 },
 	    { "GET", "/", 200 },
-	    { NULL } }, 200 },
+	    { NULL } }, 200, NULL },
 	{ { { "GET", "/302", 302 },
 	    { "GET", "/", 200 },
-	    { NULL } }, 200 },
+	    { NULL } }, 200, NULL },
 	{ { { "GET", "/303", 303 },
 	    { "GET", "/", 200 },
-	    { NULL } }, 200 },
+	    { NULL } }, 200, NULL },
 	{ { { "GET", "/307", 307 },
 	    { "GET", "/", 200 },
-	    { NULL } }, 200 },
+	    { NULL } }, 200, NULL },
 	{ { { "HEAD", "/301", 301 },
 	    { "HEAD", "/", 200 },
-	    { NULL } }, 200 },
+	    { NULL } }, 200, "551190" },
 	{ { { "HEAD", "/302", 302 },
 	    { "HEAD", "/", 200 },
-	    { NULL } }, 200 },
+	    { NULL } }, 200, "551190" },
 	/* 303 is a nonsensical response to HEAD, but some sites do
 	 * it anyway. :-/
 	 */
 	{ { { "HEAD", "/303", 303 },
 	    { "HEAD", "/", 200 },
-	    { NULL } }, 200 },
+	    { NULL } }, 200, "600830" },
 	{ { { "HEAD", "/307", 307 },
 	    { "HEAD", "/", 200 },
-	    { NULL } }, 200 },
+	    { NULL } }, 200, "551190" },
 
 	/* A non-redirecty response to a GET or HEAD should not */
 
 	{ { { "GET", "/300", 300 },
-	    { NULL } }, 300 },
+	    { NULL } }, 300, NULL },
 	{ { { "GET", "/304", 304 },
-	    { NULL } }, 304 },
+	    { NULL } }, 304, NULL },
 	{ { { "GET", "/305", 305 },
-	    { NULL } }, 305 },
+	    { NULL } }, 305, NULL },
 	{ { { "GET", "/306", 306 },
-	    { NULL } }, 306 },
+	    { NULL } }, 306, NULL },
 	{ { { "GET", "/308", 308 },
-	    { NULL } }, 308 },
+	    { NULL } }, 308, NULL },
 	{ { { "HEAD", "/300", 300 },
-	    { NULL } }, 300 },
+	    { NULL } }, 300, "551190" },
 	{ { { "HEAD", "/304", 304 },
-	    { NULL } }, 304 },
+	    { NULL } }, 304, "551190" },
 	{ { { "HEAD", "/305", 305 },
-	    { NULL } }, 305 },
+	    { NULL } }, 305, "551190" },
 	{ { { "HEAD", "/306", 306 },
-	    { NULL } }, 306 },
+	    { NULL } }, 306, "551190" },
 	{ { { "HEAD", "/308", 308 },
-	    { NULL } }, 308 },
+	    { NULL } }, 308, "551190" },
 	
 	/* Test double-redirect */
 
 	{ { { "GET", "/301/302", 301 },
 	    { "GET", "/302", 302 },
-	    { "GET", "/", 200 } }, 200 },
+	    { "GET", "/", 200 } }, 200, NULL },
 	{ { { "HEAD", "/301/302", 301 },
 	    { "HEAD", "/302", 302 },
-	    { "HEAD", "/", 200 } }, 200 },
+	    { "HEAD", "/", 200 } }, 200, "551190" },
 
 	/* POST should only automatically redirect on 301, 302 and 303 */
 
 	{ { { "POST", "/301", 301 },
 	    { "GET", "/", 200 },
-	    { NULL } }, 200 },
+	    { NULL } }, 200, "586692" },
 	{ { { "POST", "/302", 302 },
 	    { "GET", "/", 200 },
-	    { NULL } }, 200 },
+	    { NULL } }, 200, NULL },
 	{ { { "POST", "/303", 303 },
 	    { "GET", "/", 200 },
-	    { NULL } }, 200 },
+	    { NULL } }, 200, NULL },
 	{ { { "POST", "/307", 307 },
-	    { NULL } }, 307 },
+	    { NULL } }, 307, NULL },
 
 	/* Test behavior with recoverably-bad Location header */
 	{ { { "GET", "/bad", 302 },
 	    { "GET", "/bad%20with%20spaces", 200 },
-	    { NULL } }, 200 },
+	    { NULL } }, 200, "566530" },
 
 	/* Test behavior with irrecoverably-bad Location header */
 	{ { { "GET", "/bad-no-host", 302 },
-	    { NULL } }, SOUP_STATUS_MALFORMED, 302 },
+	    { NULL } }, SOUP_STATUS_MALFORMED, "528882" },
 
 	/* Test infinite redirection */
 	{ { { "GET", "/bad-recursive", 302, TRUE },
-	    { NULL } }, SOUP_STATUS_TOO_MANY_REDIRECTS },
+	    { NULL } }, SOUP_STATUS_TOO_MANY_REDIRECTS, "604383" },
 
 	/* Test redirection to a different server */
 	{ { { "GET", "/server2", 302 },
 	    { "GET", "/on-server2", 200 },
-	    { NULL } }, 200 },
+	    { NULL } }, 200, NULL },
 };
 static const int n_tests = G_N_ELEMENTS (tests);
 
@@ -163,9 +163,8 @@ do_message_api_test (SoupSession *session, TestCase *test)
 	SoupMessage *msg;
 	TestRequest *treq;
 
-	debug_printf (1, "%s %s\n",
-		      test->requests[0].method,
-		      test->requests[0].path);
+	if (test->bugref)
+		g_test_bug (test->bugref);
 
 	uri = soup_uri_new_with_base (base_uri, test->requests[0].path);
 	msg = soup_message_new_from_uri (test->requests[0].method, uri);
@@ -189,7 +188,6 @@ do_message_api_test (SoupSession *session, TestCase *test)
 	soup_test_assert_message_status (msg, test->final_status);
 
 	g_object_unref (msg);
-	debug_printf (2, "\n");
 }
 
 static void
@@ -201,15 +199,9 @@ do_request_api_test (SoupSession *session, TestCase *test)
 	TestRequest *treq;
 	GInputStream *stream;
 	GError *error = NULL;
-	guint final_status;
 
-	debug_printf (1, "%s %s\n",
-		      test->requests[0].method,
-		      test->requests[0].path);
-
-	final_status = test->request_api_final_status;
-	if (!final_status)
-		final_status = test->final_status;
+	if (test->bugref)
+		g_test_bug (test->bugref);
 
 	uri = soup_uri_new_with_base (base_uri, test->requests[0].path);
 	reqh = soup_session_request_http_uri (session,
@@ -219,7 +211,6 @@ do_request_api_test (SoupSession *session, TestCase *test)
 	g_assert_no_error (error);
 	if (error) {
 		g_error_free (error);
-		debug_printf (2, "\n");
 		return;
 	}
 
@@ -239,8 +230,9 @@ do_request_api_test (SoupSession *session, TestCase *test)
 
 	stream = soup_test_request_send (SOUP_REQUEST (reqh), NULL, 0, &error);
 
-	if (SOUP_STATUS_IS_TRANSPORT_ERROR (final_status)) {
-		g_assert_error (error, SOUP_HTTP_ERROR, final_status);
+	if (SOUP_STATUS_IS_TRANSPORT_ERROR (test->final_status) &&
+	    test->final_status != SOUP_STATUS_MALFORMED) {
+		g_assert_error (error, SOUP_HTTP_ERROR, test->final_status);
 		g_clear_error (&error);
 
 		g_assert_null (stream);
@@ -248,7 +240,6 @@ do_request_api_test (SoupSession *session, TestCase *test)
 
 		g_object_unref (msg);
 		g_object_unref (reqh);
-		debug_printf (2, "\n");
 		return;
 	}
 
@@ -257,7 +248,6 @@ do_request_api_test (SoupSession *session, TestCase *test)
 		g_error_free (error);
 		g_object_unref (msg);
 		g_object_unref (reqh);
-		debug_printf (2, "\n");
 		return;
 	}
 
@@ -270,11 +260,13 @@ do_request_api_test (SoupSession *session, TestCase *test)
 	g_clear_error (&error);
 	g_object_unref (stream);
 
-	g_assert_cmpint (msg->status_code, ==, final_status);
+	if (test->final_status == SOUP_STATUS_MALFORMED)
+		g_assert_cmpint (msg->status_code, ==, test->requests[0].status_code);
+	else
+		g_assert_cmpint (msg->status_code, ==, test->final_status);
 
 	g_object_unref (msg);
 	g_object_unref (reqh);
-	debug_printf (2, "\n");
 }
 
 static void
@@ -301,107 +293,6 @@ do_sync_req_api_test (gconstpointer test)
 	do_request_api_test (sync_session, (TestCase *)test);
 }
 
-typedef struct {
-	SoupSession *session;
-	SoupMessage *msg1, *msg2;
-	SoupURI *uri1, *uri2;
-	SoupSocket *sock1, *sock2;
-} ConnectionTestData;
-
-static void
-msg2_finished (SoupSession *session, SoupMessage *msg2, gpointer user_data)
-{
-	soup_test_assert_message_status (msg2, SOUP_STATUS_OK);
-}
-
-static void
-unpause_msg1 (SoupMessage *msg2, gpointer user_data)
-{
-       ConnectionTestData *data = user_data;
-
-       soup_test_assert (data->sock1 != NULL,
-			 "msg1 has no connection");
-       soup_test_assert (data->sock2 != NULL,
-			 "msg2 has no connection");
-       soup_test_assert (data->sock1 != data->sock2,
-			 "Both messages sharing the same connection");
-
-       soup_session_unpause_message (data->session, data->msg1);
-}
-
-static gboolean
-msg1_just_restarted (gpointer user_data)
-{
-	ConnectionTestData *data = user_data;
-
-	soup_session_pause_message (data->session, data->msg1);
-
-	data->msg2 = soup_message_new_from_uri ("GET", data->uri2);
-
-	g_signal_connect (data->msg2, "got_body",
-			  G_CALLBACK (unpause_msg1), data);
-
-	soup_session_queue_message (data->session, data->msg2, msg2_finished, data);
-	return FALSE;
-}
-
-static void
-msg1_about_to_restart (SoupMessage *msg1, gpointer user_data)
-{
-	ConnectionTestData *data = user_data;
-
-	/* Do nothing when loading the redirected-to resource */
-	if (!SOUP_STATUS_IS_REDIRECTION (data->msg1->status_code))
-		return;
-
-	/* We have to pause msg1 after the I/O finishes, but before
-	 * the queue runs again.
-	 */
-	g_idle_add_full (G_PRIORITY_HIGH, msg1_just_restarted, data, NULL);
-}
-
-static void
-request_started (SoupSession *session, SoupMessage *msg,
-		 SoupSocket *socket, gpointer user_data)
-{
-	ConnectionTestData *data = user_data;
-
-	if (msg == data->msg1)
-		data->sock1 = socket;
-	else if (msg == data->msg2)
-		data->sock2 = socket;
-	else
-		g_warn_if_reached ();
-}
-
-static void
-do_connection_test (void)
-{
-	ConnectionTestData data;
-
-	memset (&data, 0, sizeof (data));
-
-	data.session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
-	g_signal_connect (data.session, "request-started",
-			  G_CALLBACK (request_started), &data);
-
-	data.uri1 = soup_uri_new_with_base (base_uri, "/301");
-	data.uri2 = soup_uri_new_with_base (base_uri, "/");
-	data.msg1 = soup_message_new_from_uri ("GET", data.uri1);
-
-	g_signal_connect (data.msg1, "got-body",
-			  G_CALLBACK (msg1_about_to_restart), &data);
-	soup_session_send_message (data.session, data.msg1);
-
-	soup_test_assert_message_status (data.msg1, SOUP_STATUS_OK);
-
-	g_object_unref (data.msg1);
-	soup_uri_free (data.uri1);
-	soup_uri_free (data.uri2);
-
-	soup_test_session_abort_unref (data.session);
-}
-
 static void
 server_callback (SoupServer *server, SoupMessage *msg,
 		 const char *path, GHashTable *query,
@@ -504,24 +395,24 @@ main (int argc, char **argv)
 {
 	GMainLoop *loop;
 	SoupServer *server, *server2;
-	guint port;
+	SoupURI *uri2;
 	char *path;
 	int n, ret;
 
 	test_init (argc, argv, NULL);
 
-	server = soup_test_server_new (TRUE);
+	server = soup_test_server_new (SOUP_TEST_SERVER_IN_THREAD);
 	soup_server_add_handler (server, NULL,
 				 server_callback, NULL, NULL);
-	port = soup_server_get_port (server);
-	base_uri = soup_uri_new ("http://127.0.0.1");
-	soup_uri_set_port (base_uri, port);
+	base_uri = soup_test_server_get_uri (server, "http", NULL);
 
-	server2 = soup_test_server_new (TRUE);
+	server2 = soup_test_server_new (SOUP_TEST_SERVER_IN_THREAD);
 	soup_server_add_handler (server2, NULL,
 				 server2_callback, NULL, NULL);
-	server2_uri = g_strdup_printf ("http://127.0.0.1:%d/on-server2",
-				       soup_server_get_port (server2));
+	uri2 = soup_test_server_get_uri (server2, "http", NULL);
+	soup_uri_set_path (uri2, "/on-server2");
+	server2_uri = soup_uri_to_string (uri2, FALSE);
+	soup_uri_free (uri2);
 
 	loop = g_main_loop_new (NULL, TRUE);
 
@@ -531,8 +422,8 @@ main (int argc, char **argv)
 	sync_session = soup_test_session_new (SOUP_TYPE_SESSION_SYNC, NULL);
 
 	for (n = 0; n < n_tests; n++) {
-		path = g_strdup_printf ("/redirect/async/msg/%d-%s-%d", n
-					, tests[n].requests[0].method,
+		path = g_strdup_printf ("/redirect/async/msg/%d-%s-%d", n,
+					tests[n].requests[0].method,
 					tests[n].requests[0].status_code);
 		g_test_add_data_func (path, &tests[n], do_async_msg_api_test);
 		g_free (path);
@@ -556,8 +447,6 @@ main (int argc, char **argv)
 		g_free (path);
 	}
 
-	g_test_add_func ("/redirect/reuse", do_connection_test);
-
 	ret = g_test_run ();
 
 	g_main_loop_unref (loop);
